@@ -80,7 +80,7 @@ int id;
 int i,j,target;
 char *fp,* org;
 int len;
-int p[2],pp;
+int p[2];
 char ch;
 int slen;
 int er;
@@ -90,12 +90,12 @@ int pd;
 PROC_SEND ps;
 char * msg;
 char * _close_flags, * cf;
+int ret_errno;
 
 	lock_task(stream_lock);
 	for ( len = 0 ; stpp[len] ; len ++ );
 	stpfd = d_alloc((len*2+1)*sizeof(int),122);
 	_close_flags = d_alloc(len*2+1,235);
-	stpfd[len] = 0;
 	msg = copy_str("+");
 	target = 0;
 	for ( i = 0 ; i < len ; i ++ ) {
@@ -120,12 +120,14 @@ char * _close_flags, * cf;
 			}
 		}
 	}
-	len = target;
 	unlock_task(stream_lock,"launch_proc");
 	for ( i = 0 ; i < len ; i ++ ) {
 		if ( close_flags[i] )
 			s_close(stpp[i]);
 	}
+
+	len = target;
+
 	if ( strlen(msg) == 1 ) {
 		d_f_ree(msg);
 		msg = 0;
@@ -145,12 +147,13 @@ char * _close_flags, * cf;
 		argv = _argv;
 	}
 
-retry:
 	fp = d_alloc(OPEN_MAX+1,130);
 	org = d_alloc(len,131);
 	pipe_dup = d_alloc(sizeof(int)*OPEN_MAX,131);
 
 	er = pipe(p);
+retry:
+	errno = 0;
 	id = fork();
 	if ( id == 0 ) {
 		/* child process */
@@ -219,28 +222,34 @@ retry:
 		er = execvp(str,argv);
 		exit(1);
 	}
+	if ( id < 0 && errno == EAGAIN ) {
+		sleep_sec(1);
+		goto retry;
+	}
+	ret_errno = errno;
 	close(p[0]);
+
+	if ( ret_str_parent )
+		slen = strlen(ret_str_parent);
+	cf = _close_flags;
+	for ( i = 0 , _stpfd = stpfd ; i < len ; _stpfd ++ , cf ++ , i ++ ) {
+		if ( ret_str_parent )
+			write(*_stpfd,ret_str_parent,slen);
+		if ( *cf )
+			close(*_stpfd);
+	}
+
 	if ( id == -1 ) {
 		close(p[1]);
 		d_f_ree(fp);
 		d_f_ree(org);
 		d_f_ree(pipe_dup);
+		d_f_ree(stpfd);
 		if ( _argv ) {
 			d_f_ree(_argv);
 			d_f_ree(msg);
 		}
-		if ( errno == EAGAIN )
-			goto retry;
-		return - errno;
-	}
-	if ( ret_str_parent )
-		slen = strlen(ret_str_parent);
-	cf = _close_flags;
-	for ( _stpfd = stpfd ; *_stpfd ; _stpfd ++ , cf ++ ) {
-		if ( ret_str_parent )
-			write(*_stpfd,ret_str_parent,slen);
-		if ( *cf )
-			close(*_stpfd);
+		return - ret_errno;
 	}
 	errno = 0;
 	er = write(p[1],&ch,1);
