@@ -265,105 +265,6 @@ int tid;
 
 
 
-#ifndef DEADLOCK_CHECK
-void
-lock_task(SEM s)
-{
-int value;
-retry:
-	if ( _sem_getvalue(s,&value) < 0 || value > 1 )
-		er_panic("lock_task1");
-	errno = 0;
-	if ( sem_wait(&s->ss) ) {
-		if ( errno == EINTR )
-			goto retry;
-	}
-	if ( _sem_getvalue(s,&value) < 0 || value >= 1 ) {
-		printf("??????\n");
-	  	goto retry;
-		er_panic("lock_task");
-	}
-	s->tid = _get_tid();
-}
-#endif
-
-int test;
-
-#ifdef DEADLOCK_CHECK
-void
-lock_task(SEM s)
-{
-SEM sp;
-int tid;
-int value;
-extern SEM block_lock;
-
-
-	if ( s == block_lock )
-		goto retry;
-	tid = _get_tid();
-	for ( sp = sem_list ; sp ; sp = sp->next ){
-		if ( s == sp ) {
-			if ( sp->tid == tid ) {
-				fprintf(stderr,"************* DOUBLELOCK\n");
-				er_panic("lock_task");
-			}
-			break;
-		}
-		if ( sp->tid != tid )
-			continue;
-		fprintf(stderr,"************* DEADLOCK %i - %i\n",
-			sp->level,s->level);
-		er_panic("lock_task");
-	}
-retry:
-	if ( _sem_getvalue(s,&value) < 0 || value > 1 )
-		er_panic("lock_task1");
-	errno = 0;
-	if ( sem_wait(&s->s) ) {
-		if ( errno == EINTR )
-			goto retry;
-	}
-	if ( _sem_getvalue(s,&value) < 0 || value >= 1 ) {
-		printf("??????\n");
-		goto retry;
-		er_panic("lock_task");
-	}
-	s->tid = tid;
-}
-#endif
-
-#ifndef DEADLOCK_CHECK
-void
-unlock_task(SEM s,char *str)
-{
-int value;
-	s->tid = 0; 
-	sem_post(&s->ss);
-if ( _sem_getvalue(s,&value) < 0 || value >= 2 ) {
-fprintf(stderr,"*********** ???? %s %i ************\n",str,_get_tid());
- er_panic("unlock_task");
-}
-}
-#endif
-
-#ifdef DEADLOCK_CHECK
-void
-unlock_task(SEM s,char * str)
-{
-int value;
-	s->tid = 0;
-if ( _sem_getvalue(s,&value) < 0 || value  != 0 )
-printf("##########################3 %s\n",str);
-	sem_post(&s->s);
-if ( _sem_getvalue(s,&value) <0  || value >= 2 ) {
-fprintf(stderr,"*********** ???? %s %i ************\n",str,_get_tid());
- er_panic("unlock_task");
-}
-}
-#endif
-
-
 int
 get_pri(int tid)
 {
