@@ -25,6 +25,22 @@
 #include	"xl.h"
 #include	"resource.h"
 #include	"pri_level.h"
+#include	"rt_cache.h"
+
+
+#define MIN_CACHE_EXPIRE	60
+#define MAX_CACHE_EXPIRE	(30*24*3600)
+typedef struct rrr_type {
+	struct rrr_type *	next;
+	L_CHAR *		server;
+	int			port;
+	int			cnt;
+} RRR_TYPE;
+
+typedef struct purge_url {
+	struct purge_url *	next;
+	L_CHAR *		url;
+} PURGE_URL;
 
 
 MP_CACHE direct;
@@ -34,12 +50,7 @@ MP_ERR_ROUTE *	err_route_cache;
 
 SEM route_lock;
 
-typedef struct rrr_type {
-	struct rrr_type *	next;
-	L_CHAR *		server;
-	int			port;
-	int			cnt;
-} RRR_TYPE;
+PURGE_URL * purge_url_head, * purge_url_tail;
 
 RRR_TYPE * rrr_list;
 
@@ -47,9 +58,12 @@ void _flush_route(MP_ROUTE *);
 void err_route_tick();
 
 SYS_QUEUE	rr_que1,rr_que2;
+int cache_start;
 
 MP_ROUTE_T * wait_list;
 
+void rr_task_cache();
+
 void
 init_mp_cache(MP_CACHE * c)
 {
@@ -99,6 +113,7 @@ void gc_gb_sexp();
 	rr_que2.pri = PRI_FETCH;
 	setup_queue(&rr_que2);
 
+	create_task(rr_task_cache,0,0);
 	new_tick(err_route_tick,10,0);
 }
 
@@ -149,6 +164,56 @@ MP_ROUTE_T * n;
 	return n;
 }
 
+void
+_insert_purge_url(L_CHAR * url)
+{
+PURGE_URL * pm;
+	pm = d_alloc(sizeof(*pm),22);
+	pm->url = ll_copy_str(url,22);
+	pm->next = 0;
+	if ( purge_url_head ) {
+		purge_url_tail->next = pm;
+		purge_url_tail = pm;
+	}
+	else	purge_url_tail = purge_url_head = pm;
+	cache_start = 1;
+	wakeup_task((int)&cache_start);
+}
+
+void
+insert_purge_url(L_CHAR * url)
+{
+	lock_task(route_lock);
+	_insert_purge_url(url);
+	unlock_task(route_lock,"insert_purge_url");
+}
+
+L_CHAR *
+_delete_purge_url()
+{
+PURGE_URL * ret;
+L_CHAR * _ret;
+	if ( purge_url_head == 0 )
+		return 0;
+	ret = purge_url_head;
+	purge_url_head = ret->next;
+	if ( purge_url_head == 0 )
+		purge_url_tail = 0;
+	_ret = ret->url;
+	d_f_ree(ret);
+	return _ret;
+}
+
+L_CHAR *
+delete_purge_url()
+{
+L_CHAR * ret;
+	lock_task(route_lock);
+	ret = _delete_purge_url();
+	unlock_task(route_lock,"delete_purge_url");
+	return ret;
+}
+
 void
 free_mp_route_t(MP_ROUTE_T * n)
 {
@@ -412,6 +477,8 @@ MP_ROUTE * r;
 		r = cc->free_list;
 		cc->free_list = r->next;
 		insert_rring(cc,r);
+		r->flags = 0;
+		r->sts = 0;
 		return r;
 	}
 	else {
@@ -421,6 +488,8 @@ MP_ROUTE * r;
 			er_panic("_new_route");
 		free_path(r);
 		insert_rring(cc,r);
+		r->flags = 0;
+		r->sts = 0;
 		return r;
 	}
 }
@@ -494,6 +563,8 @@ int f;
 	if ( inner == 0 )
 		return 0;
 	r = _new_route(cc);
+	r->modify = inner->modify;
+	r->update = inner->update;
 	if ( f == 2 ) {
 		r->head = r->tail = 0;
 		for ( p1 = p_start ; ; p1 = p1->next ) {
@@ -555,12 +626,48 @@ MP_ROUTE * mp;
 	}
 }
 
+MP_PATH *
+copy_mp_path_list(MP_PATH * p)
+{
+MP_PATH * ret,** retp,* path;
+	ret = 0;
+	retp = &ret;
+	for ( ; p ; p = p->next ) {
+		path = d_alloc(sizeof(*path),22);
+		path->url = ll_copy_str(p->url,333);
+		path->next = 0;
+		*retp = path;
+		retp = &path->next;
+	}
+	return ret;
+}
+
+int
+cmp_mp_path(MP_PATH * a,MP_PATH * b)
+{
+int cmp;
+	for ( ; a && b ; a = a->next , b = b->next ) {
+		cmp = l_strcmp(a->url,b->url);
+		if ( cmp == 0 )
+			continue;
+		return cmp;
+	}
+	if ( a )
+		return 1;
+	if ( b )
+		return -1;
+	return 0;
+}
+
 MP_ROUTE *
 _insert_route(MP_ROUTE * r,XL_SEXP * list,L_CHAR * start,L_CHAR * target)
 {
-MP_PATH * p, * p1, * p2;
+MP_PATH * p, * p1, * p2, * old;
 XL_SEXP * rec;
 
+	if ( r->sts == RS_OK )
+		old = copy_mp_path_list(r->head);
+	else	old = 0;
 	free_path(r);
 	p = 0;
 	for ( ; get_type(list) == XLT_PAIR ; list = cdr(list) ) {
@@ -595,16 +702,30 @@ XL_SEXP * rec;
 	r->head = p;
 	for ( ; p->next ; p = p->next );
 	r->tail = p;
+	r->update = get_xltime();
+	if ( cmp_mp_path(old,r->head) == 0 ) {
+		if ( r->modify == 0 )
+			r->modify = r->update;
+	}
+	else	r->modify = r->update;
 	r->sts = RS_OK;
+	r->flags |= RF_DIRTY;
+	free_mp_path(old);
+	cache_start = 1;
+	wakeup_task((int)&cache_start);
 	return r;
 err:
 	r->sts = RS_ERR;
+	r->update = get_xltime();
+	if ( r->modify == 0 )
+		r->modify = r->update;
 	for ( ; p ; ) {
 		p1 = p;
 		p = p->next;
 		d_f_ree(p1->url);
 		d_f_ree(p1);
 	}
+	free_mp_path(old);
 	return 0;
 }
 
@@ -811,6 +932,67 @@ int _sts;
 	return ret;
 }
 
+
+MP_ROUTE *
+_get_in_file(L_CHAR * start,L_CHAR * target)
+{
+MP_ROUTE * ret,* new_r;
+
+	unlock_task(route_lock,"_get_in_file");
+	ret = get_rt_cache(start,target);
+printf("GET RT %x\n",ret);
+	lock_task(route_lock);
+	if ( ret == 0 )
+		return 0;
+	new_r = _new_route(&direct);
+	new_r->head = ret->head;
+	new_r->tail = ret->tail;
+	new_r->modify = ret->modify;
+	new_r->update = ret->update;
+	new_r->sts = RS_OK;
+	d_f_ree(ret);
+	return new_r;
+}
+
+
+MP_ROUTE *
+_resolve_in_file(MP_ROUTE ** org,L_CHAR * start,L_CHAR * target)
+{
+MP_ROUTE * ret, * new_r;
+int cmp;
+	cmp = l_strcmp(start,target);
+	switch ( cmp ) {
+	case -1:
+		*org = _get_in_file(start,target);
+		if ( *org == 0 ) {
+			ret = 0;
+			break;
+		}
+		(*org)->lock ++;
+		ret = _search_route(&direct,start,target);
+		if ( ret == 0 )
+			ret = 0;
+		else	ret->lock ++;
+		break;
+	case 0:
+		ret = 0;
+		break;
+	case 1:
+		*org = _get_in_file(target,start);
+		if ( *org == 0 ) {
+			ret = 0;
+			break;
+		}
+		(*org)->lock ++;
+		ret = _search_route(&direct,target,start);
+		if ( ret == 0 )
+			ret = 0;
+		else	ret->lock ++;
+		break;
+	}
+	return ret;
+}
+
 RRR_TYPE *
 _rrr_search(URL * u)
 {
@@ -990,6 +1172,9 @@ XL_INTERPRETER * xli;
 int ses;
 XL_SEXP * cmd;
 URL u;
+unsigned int now;
+int diff;
+MP_ROUTE * r1, * r2;
 
 	que = (SYS_QUEUE *)GET_TKEY(d);
 	key = touch_qkey(que);
@@ -1010,7 +1195,42 @@ URL u;
 			break;
 
 		lock_task(route_lock);
-		if ( n->route == 0 ) {
+		n->route = r1
+			= _resolve_in_file(&r2,n->handle1_start,n->handle1_target);
+
+		if ( n->route ) {
+			n->flags |= RQT_ROUTE_OK;
+			wakeup_task((int)n->route);
+			wakeup_task((int)n);
+			if ( (n->flags & RQT_LOCK_MASK) == RQT_UNLOCK ) {
+				if ( n->route )
+					_flush_route(n->route);
+				free_mp_route_t(n);
+			}
+			now = get_xltime();
+			diff = r2->update - r2->modify;
+			if ( diff < MIN_CACHE_EXPIRE )
+				diff = MIN_CACHE_EXPIRE;
+			else if ( diff > MAX_CACHE_EXPIRE )
+				diff = MAX_CACHE_EXPIRE;
+			if ( r2->update + diff < now ) {
+				n = new_mp_route_t(
+					r2->head->url,
+					r2->tail->url,
+					RQT_UNLOCK|RQT_ROUTE);
+				n->route = r2;
+				n->handle1_start = n->start;
+				n->handle1_target = n->target;
+				n->handle2_start = n->start;
+				n->handle2_target = n->target;
+			}
+			else {
+				/* cache is usable */
+				unlock_task(route_lock,"rr_task1");
+				continue;
+			}
+		}
+		else {
 			n->route = 
 				_new_route2(
 					&direct,
@@ -1026,6 +1246,10 @@ URL u;
 		gc_push(0,0,"rr_task1");
 
 		cmd = n_get_symbol("MPRouting");
+		if ( n->flags & RQT_ROUTE_REVERSE )
+			set_attribute(cmd,
+				l_string(std_cm,"trigger"),
+				l_string(std_cm,"on"));
 		get_url2(&u,n->handle2_start,1612);
 
 		n->get = remote_session(
@@ -1132,6 +1356,7 @@ int flags;
 			wakeup_task((int)n);
 
 			gc_pop(0,0);
+
 			goto last;
 		}
 		lock_task(route_lock);
@@ -1189,6 +1414,67 @@ ss_printf("ERR RETRY %ls %ls\n",n->handle1_start,n->handle1_target);
 
 }
 
+typedef struct mp_list {
+	struct mp_list *	next;
+	MP_ROUTE *		mp;
+} MP_LIST;
+
+void
+rr_task_cache()
+{
+MP_ROUTE * mp;
+int f;
+MP_LIST * lst, * el;
+L_CHAR * map;
+	for ( ; ; ) {
+		lock_task(route_lock);
+		for ( ; cache_start == 0 ; ) {
+			sleep_task((int)&cache_start,route_lock);
+			lock_task(route_lock);
+		}
+		cache_start = 0;
+		lst = 0;
+		for ( mp = direct.root->next ;
+				mp != direct.root ;
+				mp = mp->next ) {
+			if ( mp->sts != RS_OK )
+				continue;
+			if ( !(mp->flags & RF_DIRTY) )
+				continue;
+			mp->flags &= ~RF_DIRTY;
+			el = d_alloc(sizeof(*el),22);
+			el->mp = mp;
+			el->next = lst;	
+			lst = el;
+			mp->lock ++;
+		}
+		unlock_task(route_lock,"rr_task_cache");
+
+		for ( ; lst ; ) {
+			el = lst;
+printf("SET RT %ls %ls\n",el->mp->head->url,el->mp->tail->url);
+			set_rt_cache(el->mp);
+
+			flush_route(el->mp);
+		
+			lst = el->next;
+			d_f_ree(el);
+		}
+		flush_rt_cache();
+
+		sleep_sec(1);
+
+		for ( ; ; ) {
+			map = delete_purge_url();
+			if ( map == 0 )
+				break;
+			purge_rcache(map);
+			d_f_ree(map);
+		}
+		flush_rt_cache();
+	}
+}
+
 MP_ROUTE *
 resolve_route(L_CHAR * start,L_CHAR * target,int wait_flag)
 {
@@ -1258,13 +1544,14 @@ int sts;
 		goto end;
 	}
 	_wait_mp_route(n);
-/*
-ss_printf("STS %x\n",n->flags);
-*/
+
+ss_printf("STS %x %x\n",n,n->flags);
+
 	if ( (n->flags & RQT_ROUTE_RES_MASK) == RQT_ROUTE_ERR )
 		ret = MP_NO_ROUTE;
 	else	ret = n->route;
-	free_mp_route_t(n);
+	if ( (n->flags & RQT_LOCK_MASK) == RQT_LOCK )
+		free_mp_route_t(n);
 end:
 	if ( ret == MP_NO_ROUTE )
 		_ret = MP_NO_ROUTE;
@@ -1419,6 +1706,7 @@ MAP_HISTORY * ret, * ret1;
 URL uu;
 RESOURCE * r;
 int len;
+int err_flag;
 
 LOAD_RESOURCE_WORK * w, * wp;
 
@@ -1485,6 +1773,7 @@ set_cpu_msg(2003);
 
 	ret = 0;
 	wp = w;
+	err_flag = 0;
 	for ( crd_prev = mpr->head ; ;
 			crd_prev = crd_next ) {
 		if ( crd_prev == 0 ) {
@@ -1508,17 +1797,40 @@ set_cpu_msg(2003);
 		get_url2(&uu,map->url,1614);
 		r = get_lrw(w,&uu);
 		free_url(&uu);
-		if ( r == 0 )
-			goto err;
+		if ( r == 0 ) {
+			err_flag = 1;
+			insert_purge_url(map->url);
+			goto nx1;
+		}
 		get_url2(&uu,crd_prev->url,1615);
 		if ( url_cmp(&uu,&r->map.src) == 0 ) {
+			get_url2(&uu,crd_next->url,122);
+			if ( url_cmp(&uu,&r->map.dest) ) {
+				err_flag = 1;
+				insert_purge_url(map->url);
+				goto nx1;
+			}
 			ret = cons_mh(r,MHD_REVERSE,ret);
 		}
 		else {
+			if ( url_cmp(&uu,&r->map.dest) ) {
+				err_flag = 1;
+				insert_purge_url(map->url);
+				goto nx1;
+			}
+			get_url2(&uu,crd_next->url,222);
+			if ( url_cmp(&uu,&r->map.src) ) {
+				err_flag = 1;
+				insert_purge_url(map->url);
+				goto nx1;
+			}
 			ret = cons_mh(r,MHD_FORWARD,ret);
 		}
+	nx1:
 		free_url(&uu);
 	}
+	if ( err_flag )
+		goto err;
 
 set_cpu_msg(2004);
 	free_lrw_req_next(w);
