@@ -87,9 +87,9 @@ set_t_msg(55);
 	wf->mh = copy_mh(mh);
 	wf->v_reso = v_reso;
 	wf->flags = WFF_ACTIVE|WFF_LUSTER|WFF_DIRTY;
-
-	draw->c.weight = W_E2D_LUSTER;
-
+/*
+	draw->c.weight = 0;
+*/
 	wf->pixels = d_alloc(env.win_width * env.win_height * sizeof(long),84);
 	for ( i = 0 ; i < env.win_width * env.win_height ; i ++ )
 		wf->pixels[i] = C_TRANSPARENT|C_DIRTY;
@@ -113,7 +113,12 @@ set_t_msg(55);
 	wf->next = *wfp;
 	*wfp = wf;
 	wf->lock = 0;
+	wf->lock_list = 0;
+	wf->browse = 0;
 	wf->aboat_count = 0;
+
+	wf->insert_time = get_xltime();
+
 	_wakeup_wf(wf);
 end:
 	*ret_wfid = wf->id;
@@ -150,6 +155,7 @@ RESOURCE * r;
 	if ( *wfp )
 		*wfp = wf->next;
 	wakeup_task((int)wf);
+	wf_free_lock_list(wf);
 	d_f_ree(wf);
 	_wf_flame_redraw();
 	call_gv_event(RID_INDICATED_COORD,l_string(std_cm,"hide"));
