@@ -114,6 +114,7 @@ _win_lock(WIN_FLAME * wf)
 {
 int id;
 int _wf_redraw_pri();
+WF_LOCK_LIST * wfl;
 
 retry:
 	if ( wf->flags & WFF_FREE )
@@ -122,6 +123,11 @@ retry:
 		return -1;
 	if ( wf->lock >= 0 ) {
 		wf->lock ++;
+
+		wfl = d_alloc(sizeof(*wfl),33);
+		wfl->next = wf->lock_list;
+		wfl->tid = get_tid();
+		wf->lock_list  = wfl;
 	}
 	else {
 		id = wf->id;
@@ -140,6 +146,7 @@ _win_wlock(WIN_FLAME * wf)
 {
 int id;
 int _wf_redraw_pri();
+WF_LOCK_LIST * wfl;
 	for ( ; wf->lock ; ) {
 		id = wf->id;
 		sleep_task((int)wf,env.wf_lock);
@@ -149,6 +156,12 @@ set_t_msg(4);
 			return -1;
 	}
 	wf->lock = -1;
+
+	wfl = d_alloc(sizeof(*wfl),33);
+	wfl->next = wf->lock_list;
+	wfl->tid = get_tid();
+	wf->lock_list  = wfl;
+
 	wakeup_task((int)_wf_redraw_pri);
 	return 0;
 }
@@ -158,7 +171,20 @@ _win_unlock(WIN_FLAME * wf)
 {
 WIN_FLAME * wf2;
 int _wf_redraw_pri();
+WF_LOCK_LIST ** wflp, * wfl;
+int tid;
 
+	tid = get_tid();
+	for ( wflp = &wf->lock_list ; *wflp ; wflp = &(*wflp)->next ) {
+		if ( (*wflp)->tid != tid )
+			continue;
+		wfl = *wflp;
+		*wflp = wfl->next;
+		d_f_ree(wfl);
+		goto ok;
+	}
+	return;
+ok:
 	if ( wf->lock == 0 )
 		er_panic("_win_unlock");
 	if ( wf->lock > 0 )
@@ -170,6 +196,18 @@ int _wf_redraw_pri();
 	}
 }
 
+void
+wf_free_lock_list(WIN_FLAME * wf)
+{
+WF_LOCK_LIST * wfl;
+	for ( ; wf->lock_list ; ) {
+		wfl = wf->lock_list;
+		wf->lock_list = wfl->next;
+		d_f_ree(wfl);
+	}
+}
+
+
 int
 win_lock(WIN_FLAME * wf)
 {
@@ -887,7 +925,7 @@ set_t_msg(21);
 int
 _wf_set_current(WIN_FLAME * now,WIN_FLAME * target)
 {
-WIN_FLAME * c, * wf, * oc;
+WIN_FLAME * c, * wf;
 MAP_HISTORY * base, * mh;
 GB_POINT center,_center[3];
 REAL1 reso,_reso[3];
@@ -900,18 +938,17 @@ REAL1 _t;
 
 	if ( env.flame == 0 )
 		return 0;
+
 	if ( now && env.flame == now && now->next == 0 ) {
-		env.flame_base = 0;
 		return 0;
 	}
-	for ( oc = env.flame ; oc && oc->mh ; oc = oc->next );
-	if ( oc == 0 )
-		er_panic("_wf_set_current");
-	if ( target == oc )
-		return 0;
+	if ( target && target == env.flame_base )
+		return 1;
 	c = 0;
 	calc_pitch(0); 
 	for ( wf = env.flame ; wf ; wf = wf->next ) {
+		if ( wf == env.flame_base )
+			continue;
 		if ( wf == now )
 			continue;
 		if ( wf->flags & WFF_FREE )
@@ -973,8 +1010,8 @@ printf("*********SET  %f %f %f %f %f\n",reso,center.x,center.y,env.flame_base_ro
 	}
 	if ( c == 0 )
 		return 0;
-	if ( oc == c )
-		return 0;
+	if ( env.flame_base == c )
+		er_panic("wf_set_current");
 	base = rev_mh(c->mh);
 	for ( wf = env.flame ; wf ; wf = wf->next ) {
 		if ( wf == c ) {
