@@ -22,7 +22,7 @@
 
 #include	"long_char.h"
 
-#define CLOSE_ZONBIE	1000
+#define CLOSE_ZONBIE	10000
 #define INITIAL_INTERVAL	10
 #define INITIAL_THPUT		1000
 
@@ -79,6 +79,9 @@ typedef struct s_header {
 
 	char		pushed;
 	char		pushed_flag;
+
+	unsigned int	close_time;
+	unsigned int	zonbie_access;
 } S_HEADER;
 
 typedef struct s_string {
@@ -174,7 +177,16 @@ STREAM * s_connect_XLoHTTP(
 
 void init_stream();
 void s_open(STREAM *,int);
+/*
 int s_close(STREAM *);
+*/
+
+int __d_s_close(STREAM *,char *,int);
+#define _s_close(st)	__d_s_close(st,__FILE__,__LINE__)
+
+int _d_s_close(STREAM *,char *,int);
+#define s_close(st)	_d_s_close(st,__FILE__,__LINE__)
+
 int s_write(STREAM *,void *,int);
 int s_read(STREAM *,void *,int);
 int s_printf(STREAM *,char *,...);
