@@ -35,8 +35,12 @@
 
 void init_task(int);
 SEM new_lock(int level);
-void lock_task(SEM);
-void unlock_task(SEM,char *);
+void _lock_task(SEM,char *,int);
+void _unlock_task(SEM,char *,char *, int);
+
+#define lock_task(s)	_lock_task((s),__FILE__,__LINE__)
+#define unlock_task(s,str)	_unlock_task((s),(str),__FILE__,__LINE__)
+
 void rw_lock_init(RW_LOCK *,int);
 void read_lock(RW_LOCK *);
 void write_lock(RW_LOCK *);
