@@ -166,5 +166,9 @@ bl_new_connection(
 	void * work);
 
 void gcv_url(URL *);
+char * change_delim_str(char * buf);
+L_CHAR * change_delim_l_char(L_CHAR * buf);
+char * normalize_delim_str(char * buf);
+
 
 #endif
