@@ -379,7 +379,6 @@ HTTP_INFO o_info;
 
 		if ( e->flags & XoH_E_KEEP_ALIVE )
 {
-printf("insert_ss\n");
 			_insert_ss(lst->con,e->ip,e->xli,e->pl);
 }
 		else	s_close(lst->con);
@@ -388,7 +387,6 @@ printf("insert_ss\n");
 	}
 	else {
 		e = _search_XoH_entry(lst->h.cockie);
-printf("search_xoh_entry %x\n",e);
 		if ( e == 0 ) {
 		XLoHTTP_HEADER h;
 
@@ -430,7 +428,6 @@ ss_task()
 STREAM * new;
 STREAM_STOCK * ss;
 int ret;
-printf("start ss_task\n");
 	lock_task(XoH_lock);
 	ss_run_tasks ++;
 	unlock_task(XoH_lock,"ss_task");
@@ -447,7 +444,6 @@ printf("start ss_task\n");
 			ss_run_tasks --;
 			ss_wait_tasks ++;
 			sleep_task((int)&ss_list,XoH_lock);
-printf("*** ss wakeup ***\n");
 			lock_task(XoH_lock);
 			ss_wait_tasks --;
 			ss_run_tasks ++;
@@ -458,18 +454,15 @@ printf("*** ss wakeup ***\n");
 		ret = new_accept(&new,&ss->key,ss->xli,ss->pl);
 		switch ( ret ) {
 		case -1:
-printf("* error *\n");
 			/* error */
 			s_close(ss->key.s);
 			d_f_ree(ss);
 			break;
 		case 0:
-printf("* nothing *\n");
 			/* nothing to do */
 			d_f_ree(ss);
 			break;
 		case 1:
-printf("* launch *\n");
 			/* launch new stream */
 			launch_accept_thread(
 				new,
@@ -479,7 +472,7 @@ printf("* launch *\n");
 			d_f_ree(ss);
 			break;
 		default:
-printf("*** panic !!! ***\n");
+			{}
 		}
 	}
 }
@@ -555,7 +548,6 @@ int len;
 		for ( ; (lst = get_recv_seq_list(e)) == 0 &&
 				(e->flags & (XoH_E_EXIT|XoH_E_F_ERR))
 					 == 0 ; ) {
-printf("recv sleep...\n");
 			sleep_task((int)e,XoH_lock);
 			lock_task(XoH_lock);
 		}
@@ -598,6 +590,7 @@ printf("recv sleep...\n");
 		last:
 			lock_task(XoH_lock);
 			e->flags |= XoH_E_EXIT_RECV;
+			e->recv_target = 0;
 			s_close(e->pipe);
 			e->pipe = 0;
 			if ( (e->flags & XoH_E_EXIT)
